### models.py
from sklearn.ensemble import (
    RandomForestClassifier,
    GradientBoostingClassifier,
    ExtraTreesClassifier,
    AdaBoostClassifier,
)
from sklearn.tree import DecisionTreeClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.svm import SVC
from sklearn.neighbors import KNeighborsClassifier
from sklearn.naive_bayes import GaussianNB
from sklearn.neural_network import MLPClassifier
from xgboost import XGBClassifier
from lightgbm import LGBMClassifier
from catboost import CatBoostClassifier


def get_models():
    return {
        "Random Forest": RandomForestClassifier(),
        "Gradient Boosting": GradientBoostingClassifier(),
        "Extra Trees": ExtraTreesClassifier(),
        "AdaBoost": AdaBoostClassifier(),
        "Decision Tree": DecisionTreeClassifier(),
        "Logistic Regression": LogisticRegression(max_iter=500),  # Increased max_iter
        "Support Vector Machine": SVC(probability=True),
        "K-Nearest Neighbors": KNeighborsClassifier(),
        "Naive Bayes": GaussianNB(),
        "Neural Network": MLPClassifier(max_iter=500),  # Increased max_iter
        "XGBoost": XGBClassifier(use_label_encoder=False, eval_metric="logloss"),
        "LightGBM": LGBMClassifier(),
        "CatBoost": CatBoostClassifier(verbose=0),
    }


def get_hyperparameter_grids():
    return {
        "Random Forest": {"n_estimators": [50, 100, 200], "max_depth": [None, 10, 20]},
        "Gradient Boosting": {
            "n_estimators": [50, 100, 200],
            "learning_rate": [0.01, 0.1, 0.2],
        },
        "Extra Trees": {"n_estimators": [50, 100, 200], "max_depth": [None, 10, 20]},
        "AdaBoost": {"n_estimators": [50, 100, 200], "learning_rate": [0.01, 0.1, 0.2]},
        "Decision Tree": {"max_depth": [None, 10, 20], "min_samples_split": [2, 5, 10]},
        "Logistic Regression": {"C": [0.1, 1, 10]},
        "Support Vector Machine": {"C": [0.1, 1, 10], "kernel": ["linear", "rbf"]},
        "K-Nearest Neighbors": {"n_neighbors": [3, 5, 10]},
        "Neural Network": {
            "hidden_layer_sizes": [(50,), (100,)],
            "alpha": [0.0001, 0.01],
        },
        "XGBoost": {"n_estimators": [50, 100, 200], "learning_rate": [0.01, 0.1, 0.2]},
        "LightGBM": {"n_estimators": [50, 100, 200], "learning_rate": [0.01, 0.1, 0.2]},
        "CatBoost": {"iterations": [50, 100, 200], "learning_rate": [0.01, 0.1, 0.2]},
    }



### preprocessing.py
import numpy as np
from sklearn.preprocessing import StandardScaler, OneHotEncoder, LabelEncoder
from sklearn.impute import SimpleImputer
from sklearn.pipeline import Pipeline
from sklearn.compose import ColumnTransformer
from sklearn.model_selection import train_test_split, StratifiedKFold


def preprocess_data(data, target_column):
    X = data.drop(columns=[target_column])
    y = data[target_column]

    # Normalize class labels to ensure they start from 0 and are continuous
    label_encoder = LabelEncoder()
    y = label_encoder.fit_transform(y)

    num_features = X.select_dtypes(include=[np.number]).columns
    cat_features = X.select_dtypes(exclude=[np.number]).columns

    num_pipeline = Pipeline(
        [("imputer", SimpleImputer(strategy="mean")), ("scaler", StandardScaler())]
    )

    cat_pipeline = Pipeline(
        [
            ("imputer", SimpleImputer(strategy="most_frequent")),
            (
                "encoder",
                OneHotEncoder(handle_unknown="ignore", sparse_output=False),
            ),  # Convert to dense
        ]
    )

    preprocessor = ColumnTransformer(
        [("num", num_pipeline, num_features), ("cat", cat_pipeline, cat_features)]
    )

    X_preprocessed = preprocessor.fit_transform(X)
    X_preprocessed = np.asarray(X_preprocessed)  # Ensure dense output

    # Use stratified split to preserve class distribution
    skf = StratifiedKFold(n_splits=5, shuffle=True, random_state=42)
    for train_index, test_index in skf.split(X_preprocessed, y):
        X_train, X_test = X_preprocessed[train_index], X_preprocessed[test_index]
        y_train, y_test = y[train_index], y[test_index]
        break  # Use only the first split

    return X_train, X_test, y_train, y_test



import pandas as pd
import numpy as np
from automlbench import preprocess_data


def test_preprocess_data():
    # Create a mock dataset
    data = pd.DataFrame(
        {
            "Feature1": [1, 2, np.nan, 4, 5],
            "Feature2": ["A", "B", "A", "B", "C"],
            "Target": [0, 1, 0, 1, 0],
        }
    )

    # Run preprocessing
    X_train, X_test, y_train, y_test = preprocess_data(data, "Target")

    # Ensure no missing values
    assert not np.isnan(X_train).any()
    assert not np.isnan(X_test).any()

    # Check label encoding
    assert set(y_train) == {0, 1}
    assert set(y_test) == {0, 1}



### utils.py
import time


def time_execution(func):
    def wrapper(*args, **kwargs):
        start_time = time.time()
        result = func(*args, **kwargs)
        end_time = time.time()
        print(
            f"Execution time for {func.__name__}: {end_time - start_time:.4f} seconds"
        )
        return result

    return wrapper


def log_message(message):
    print(f"[AutoMLBench]: {message}")



### data_loader.py
import os
import pandas as pd


def load_data(data_path):
    ext = os.path.splitext(data_path)[1].lower()
    if ext == ".csv":
        return pd.read_csv(data_path)
    elif ext in [".xlsx", ".xls"]:
        return pd.read_excel(data_path)
    elif ext == ".json":
        return pd.read_json(data_path)
    elif ext == ".parquet":
        return pd.read_parquet(data_path)
    elif ext == ".hdf5":
        return pd.read_hdf(data_path)
    else:
        raise ValueError("Unsupported file format")



import pandas as pd
import numpy as np
from automlbench import get_models, train_models


def test_train_models():
    # Create synthetic dataset
    X_train = np.random.rand(100, 5)
    X_test = np.random.rand(20, 5)
    y_train = np.random.choice([0, 1], size=100)
    y_test = np.random.choice([0, 1], size=20)

    # Train models
    results = train_models(X_train, X_test, y_train, y_test)

    # Ensure results are generated
    assert isinstance(results, dict)
    assert "Accuracy" in results[list(results.keys())[0]]  # Check accuracy is included



### evaluation.py
from sklearn.metrics import (
    accuracy_score,
    precision_score,
    recall_score,
    f1_score,
    roc_auc_score,
)
from sklearn.preprocessing import label_binarize
import numpy as np


def evaluate_model(model, X_test, y_test):
    y_pred = model.predict(X_test)

    metrics = {
        "Accuracy": accuracy_score(y_test, y_pred),
        "Precision": precision_score(
            y_test, y_pred, average="weighted", zero_division=1
        ),
        "Recall": recall_score(y_test, y_pred, average="weighted", zero_division=1),
        "F1 Score": f1_score(y_test, y_pred, average="weighted", zero_division=1),
    }

    if hasattr(model, "predict_proba"):
        if len(set(y_test)) > 2:  # Multiclass case
            try:
                metrics["AUC-ROC"] = roc_auc_score(
                    y_test, model.predict_proba(X_test), multi_class="ovr"
                )
            except ValueError:
                metrics["AUC-ROC"] = (
                    np.nan
                )  # Handle cases where AUC-ROC cannot be computed
        else:  # Binary case
            metrics["AUC-ROC"] = roc_auc_score(
                y_test, model.predict_proba(X_test)[:, 1]
            )

    return metrics



### visualization.py
import matplotlib.pyplot as plt
import pandas as pd


def plot_performance(results, metrics=["Accuracy"]):
    results_df = pd.DataFrame(results).T
    results_df = results_df[metrics]
    results_df.plot(kind="bar", figsize=(12, 6))
    plt.title("Model Performance Comparison")
    plt.ylabel("Score")
    plt.xticks(rotation=45)
    plt.legend(title="Metrics")
    plt.grid(axis="y")
    plt.show()



from setuptools import setup, find_packages

setup(
    name="automlbench",
    version="0.1.5",
    author="Ann Naser Nabil",
    author_email="ann.n.nabil@gmail.com",
    description="A Python package for automated ML model benchmarking and comparison",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/AnnNaserNabil/automlbench",
    packages=find_packages(),
    install_requires=[
        "numpy",
        "pandas",
        "scikit-learn",
        "matplotlib",
        "seaborn",
        "xgboost",
        "lightgbm",
        "shap",
        "lime",
        "catboost",
        "imbalanced-learn"

    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    python_requires=">=3.7",
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "automlbench=automlbench.cli:main"  # If you have a CLI tool
        ]
    },
)



import numpy as np
from sklearn.ensemble import RandomForestClassifier
from automlbench import evaluate_model


def test_evaluate_model():
    # Create dummy dataset
    X_test = np.random.rand(20, 5)
    y_test = np.random.choice([0, 1], size=20)

    # Train model
    model = RandomForestClassifier().fit(X_test, y_test)

    # Evaluate model
    metrics = evaluate_model(model, X_test, y_test)

    # Ensure all expected metrics exist
    assert "Accuracy" in metrics
    assert "Precision" in metrics
    assert "Recall" in metrics
    assert "F1 Score" in metrics
